package client

import (
	"bytes"
	"mime/multipart"
	"net/http"
	"random.chars.jp/git/image-board/api"
	"random.chars.jp/git/image-board/store"
	"strconv"
)

// Images returns a slice of snowflakes of all images. Only available to privileged users.
func (r *Remote) Images() ([]string, error) {
	var flakes []string
	err := r.fetch(http.MethodGet, api.Image, &flakes, nil)
	return flakes, err
}

// ImageAdd adds an image to Remote and returns a store.Image.
func (r *Remote) ImageAdd(data []byte) (store.Image, error) {
	buf := &bytes.Buffer{}
	w := multipart.NewWriter(buf)
	if f, err := w.CreateFormFile("image", ""); err != nil {
		return store.Image{}, err
	} else {
		if _, err = f.Write(data); err != nil {
			return store.Image{}, err
		}
	}

	if err := w.Close(); err != nil {
		return store.Image{}, err
	}

	if resp, err := r.request(http.MethodPost, api.Image, buf); err != nil {
		return store.Image{}, err
	} else {
		var image store.Image
		err = unmarshal(resp.Body, &image)
		return image, err
	}
}

// Image returns store.Image with given snowflake.
func (r *Remote) Image(flake string) (store.Image, error) {
	var image store.Image
	err := r.fetch(http.MethodGet, populateField(api.ImageField, "flake", flake), &image, nil)
	return image, err
}

// ImageUpdate updates metadata of store.Image with given snowflake. To persist original value in a field set \000.
func (r *Remote) ImageUpdate(flake, source, commentary, commentaryTranslation string) error {
	payload := api.ImageUpdatePayload{
		Source:                source,
		Commentary:            commentary,
		CommentaryTranslation: commentaryTranslation,
	}
	return r.requestJSONnoResp(http.MethodPatch, populateField(api.ImageField, "flake", flake), payload)
}

// ImageDestroy destroys a store.Image with given snowflake.
func (r *Remote) ImageDestroy(flake string) error {
	return r.requestNoResp(http.MethodDelete, populateField(api.ImageField, "flake", flake), nil)
}

// ImageFile returns a slice of bytes of the image with given snowflake.
func (r *Remote) ImageFile(flake string, preview bool) ([]byte, error) {
	switch preview {
	case true:
		return r.fetchAll(http.MethodGet, populateField(api.ImagePreview, "flake", flake), nil)
	case false:
		return r.fetchAll(http.MethodGet, populateField(api.ImageFile, "flake", flake), nil)
	default:
		return nil, nil
	}
}

// ImageTag returns a slice of tags of an image with given snowflake.
func (r *Remote) ImageTag(flake string) ([]string, error) {
	var tags []string
	err := r.fetch(http.MethodGet, populateField(api.ImageTag, "flake", flake), &tags, nil)
	return tags, err
}

// ImageTagAdd adds a tag to an image with given snowflake.
func (r *Remote) ImageTagAdd(flake, tag string) error {
	return r.requestNoResp(http.MethodPut,
		populateField(populateField(api.ImageTagField,
			"flake", flake),
			"tag", tag),
		nil)
}

// ImageTagRemove removes a tag from an image with given snowflake.
func (r *Remote) ImageTagRemove(flake, tag string) error {
	return r.requestNoResp(http.MethodDelete,
		populateField(populateField(api.ImageTagField,
			"flake", flake),
			"tag", tag),
		nil)
}

// ImagePages returns total amount of image pages.
func (r *Remote) ImagePages() (int, error) {
	if payload, err := r.fetchAll(http.MethodGet, api.ImagePage, nil); err != nil {
		return 0, err
	} else {
		var pages int
		if pages, err = strconv.Atoi(string(payload)); err != nil {
			return 0, err
		} else {
			return pages, nil
		}
	}
}

// ImagePage returns a slice of snowflakes of images in a page.
func (r *Remote) ImagePage(entry int) ([]string, error) {
	var flakes []string
	err := r.fetch(http.MethodGet, populateField(api.ImagePageField, "entry", strconv.Itoa(entry)), &flakes, nil)
	return flakes, err
}

// ImagePageImage returns a slice of store.Image in a page.
func (r *Remote) ImagePageImage(entry int) ([]store.Image, error) {
	var images []store.Image
	err := r.fetch(http.MethodGet, populateField(api.ImagePageImage, "entry", strconv.Itoa(entry)), &images, nil)
	return images, err
}

// ImageSearch searches for images that contains all specified tags.
func (r *Remote) ImageSearch(tags []string) ([]string, error) {
	t := tags[0]
	for i := 1; i < len(tags); i++ {
		t += "!" + tags[i]
	}
	var flakes []string
	err := r.fetch(http.MethodGet, populateField(api.SearchField, "tags", t), &flakes, nil)
	return flakes, err
}
